@@ -9,25 +9,31 @@ File::Assets - Manage .css and .js assets in a web application
 
 =head1 VERSION
 
-Version 0.056
+Version 0.061
 
 =cut
 
-our $VERSION = '0.056';
+our $VERSION = '0.061';
 
 =head1 SYNOPSIS
 
     use File::Assets
 
-    my $assets = File::Assets->new( base => [ $uri_root, $htdocs_root ] );
+    my $assets = File::Assets->new( base => [ $uri_root, $dir_root ] )
 
-    $assets->include("/static/style.css"); # File::Assets will automatically detect the type based on the extension
+    $assets->set_output_path("built/") # Put minified files in $dir_root/built/... (the trailing slash is important)
+
+    $assets->include("/static/style.css") # File::Assets will automatically detect the type based on the extension
 
     # Then, later ...
     
-    $assets->include("/static/main.js");
-    $assets->include("/static/style.css"); # This asset won't get included twice, as File::Assets will ignore repeats of a path
+    $assets->include("/static/main.js")
+    $assets->include("/static/style.css") # This asset won't get included twice, as File::Assets will ignore repeats of a path
 
+    # You can also include external assets:
+
+    $assets->include("http://ajax.googleapis.com/ajax/libs/jquery/1.2.6/jquery.min.js");
+    
     # And then, in your .tt (Template Toolkit) files:
 
     [% WRAPPER page.tt %]
@@ -68,12 +74,16 @@ our $VERSION = '0.056';
 
         print $asset->uri, "\n";
     }
-    
+
+Use the minify option to perform minification before export
+
+    my $assets = File::Assets->new( minify => 1, ... )
+
 =head1 DESCRIPTION
 
 File::Assets is a tool for managing JavaScript and CSS assets in a (web) application. It allows you to "publish" assests in one place after having specified them in different parts of the application (e.g. throughout request and template processing phases).
 
-This package has the added bonus of assisting with minification and filtering of assets. Support is built-in for YUI Compressor (L<http://developer.yahoo.com/yui/compressor/>), L<JavaScript::Minifier>, and L<CSS::Minifier>. Filtering is fairly straightforward to implement, so it's a good place to start if need a JavaScript or CSS preprocessor (e.g. something like HAML L<http://haml.hamptoncatlin.com/>)
+This package has the added bonus of assisting with minification and filtering of assets. Support is built-in for YUI Compressor (L<http://developer.yahoo.com/yui/compressor/>), L<JavaScript::Minifier>, L<CSS::Minifier>, L<JavaScript::Minifier::XS>, and L<CSS::Minifier::XS>.
 
 File::Assets was built with L<Catalyst> in mind, although this package is framework agnostic. Look at L<Catalyst::Plugin::Assets> for an easy way to integrate File::Assets with Catalyst.
 
@@ -114,33 +124,100 @@ By default, File::Assets will avoid re-minifying assets if nothing in the files
 
 You can change the name of the assets at anytime before exporting.
 
+=head2 YUI Compressor 2.2.5 is required
+
+If you want to use the YUI Compressor, you should have version 2.2.5 or above. 
+
+YUI Compressor 2.1.1 (and below) will *NOT WORK*
+
+=head2 Specifying an C<output_path> pattern
+
+When aggregating or minifying assets, you need to put the result in a new file. 
+
+You can use the following directives when crafting a path/filename pattern:
+
+    %n      The name of the asset, "assets" by default
+    %e      The extension of the asset (e.g. css, js)
+    %f      The fingerprint of the asset collection (a hexadecimal digest of the concatenated digest of each asset in the collection)
+    %k      The kind of the asset (e.g. css-screen, css, css-print, js)
+    %h      The kind head-part of the asset (e.g. css, js)
+    %l      The kind tail-part of the asset (e.g. screen, print) (essentially the media type of a .css asset)
+
+In addition, in each of the above, a ".", "/" or "-" can be placed in between the "%" and directive character.
+This will result in a ".", "/", or "-" being prepended to the directive value.
+
+The default pattern is:
+
+    %n%-l%-f.%e
+
+A pattern of C<%n%-l.%e> can result in the following:
+
+    assets.css          # name of "assets", no media type, an asset type of CSS (.css)
+    assets-screen.css   # name of "assets", media type of "screen", an asset type of CSS (.css)
+    assets.js           # name of "assets", an asset type of JavaScript (.js)
+
+If the pattern ends with a "/", then the default pattern will be appended
+
+If the pattern does not have an extension-like ending, then "%.e" will be appended
+
+    xyzzy/          => xyzzy/%n%-l-%f.%e
+    xyzzy           => xyzzy.%e
+
+=head2 Strange output or "sticky" content
+
+File::Assets uses built-in caching to share content across different objects (via File::Assets::Cache). If you're having problems
+try disabling the cache by passing "cache => 0" to File::Assets->new
+
 =head1 METHODS
 
 =cut
 
+# If the pattern does NOT begin with a "/", then the base dir will be prepended
+
 use strict;
 use warnings;
 
+use Object::Tiny qw/cache registry _registry_hash rsc filter_scheme output_path_scheme output_asset_scheme/;
+use File::Assets::Carp;
+
 use Tie::LLHash;
-use File::Assets::Asset;
-use Object::Tiny qw/registry _registry_hash rsc filter_scheme output_path_scheme output_asset_scheme/;
 use Path::Resource;
-use File::Assets::Kind;
-use File::Assets::Bucket;
 use Scalar::Util qw/blessed refaddr/;
-use Carp::Clan qw/^File::Assets::/;
 use HTML::Declare qw/LINK SCRIPT STYLE/;
 
-=head2 File::Assets->new( base => <base> )
+use File::Assets::Asset;
+use File::Assets::Cache;
+use File::Assets::Kind;
+use File::Assets::Bucket;
 
-Create and return a new File::Assets object. <base> can be:
+=head2 File::Assets->new( base => <base>, output_path => <output_path>, minify => <minify> )
+
+Create and return a new File::Assets object.
+
+You can configure the object with the following:
     
-* An array (list reference) where <base>[0] is a URI object or uri-like string (e.g. "http://www.example.com")
-and <base>[1] is a Path::Class::Dir object or a dir-like string (e.g. "/var/htdocs")
+    base            # A hash reference with a C<uri> key/value and a C<dir> key/value.
+                      For example: { uri => http://example.com/assets, dir => /var/www/htdocs/assets }
+    
+                    # A L<URI::ToDisk> object
 
-* A L<URI::ToDisk> object
+                    # A L<Path::Resource> object
 
-* A L<Path::Resource> object
+    minify          # "1" or "best" - Will either use L<JavaScript::Minifier::XS> & L<CSS::Minifier::XS> or
+                                      L<JavaScript::Minifier> & L<CSS::Minifier> (depending on availability)
+                                      for minification
+
+                    # "0" or "" or undef - Don't do any minfication (this is the default)
+
+                    # "./path/to/yuicompressor.jar" - Will use YUI Compressor via the given .jar for minification
+
+                    # "minifier" - Will use L<JavaScript::Minifier> & L<CSS::Minifier> for minification
+
+                    # "xs" or "minifier-xs" - Will use L<JavaScript::Minifier::XS> & L<CSS::Minifier::XS> for minification
+
+    output_path     # Designates the output path for minified .css and .js assets
+                      The default output path pattern is C<%n%-l%-d.%e> (rooted at the dir of <base>)
+                      See above in "Specifying an C<output_path> pattern" for details
 
 =cut
 
@@ -148,26 +225,44 @@ sub new {
     my $self = bless {}, shift;
     local %_ = @_;
 
-    my $rsc = File::Assets::Util->parse_rsc($_{rsc} || $_{base_rsc} || $_{base});
-    $rsc->uri($_{uri} || $_{base_uri}) if $_{uri} || $_{base_uri};
-    $rsc->dir($_{dir} || $_{base_dir}) if $_{dir} || $_{base_dir};
-    $rsc->path($_{base_path}) if $_{base_path};
-    $self->{rsc} = $rsc;
+    $self->set_base($_{rsc} || $_{base_rsc} || $_{base});
+    $self->set_base_uri($_{uri} || $_{base_uri}) if $_{uri} || $_{base_uri};
+    $self->set_base_dir($_{dir} || $_{base_dir}) if $_{dir} || $_{base_dir};
+    $self->set_base_path($_{base_path}) if $_{base_path};
+
+    $self->set_output_path($_{output_path} || $_{output_path_scheme} || []);
+
+    $self->name($_{name});
+    
+    $_{cache} = 1 unless exists $_{cache};
+    $self->set_cache($_{cache}) if $_{cache};
+
+#    my $rsc = File::Assets::Util->parse_rsc($_{rsc} || $_{base_rsc} || $_{base});
+#    $rsc->uri($_{uri} || $_{base_uri}) if $_{uri} || $_{base_uri};
+#    $rsc->dir($_{dir} || $_{base_dir}) if $_{dir} || $_{base_dir};
+#    $rsc->path($_{base_path}) if $_{base_path};
+#    $self->{rsc} = $rsc;
 
     my %registry;
     $self->{registry} = tie(%registry, qw/Tie::LLHash/, { lazy => 1 });
     $self->{_registry_hash} = \%registry;
 
-    $self->{name} = $_{name};
-
-    $self->{output_asset_scheme} = $_{output_asset} || $_{output_asset_scheme} || [];
-    $self->{output_path_scheme} = $_{output_path} || $_{output_path_scheme} || [];
     $self->{filter_scheme} = {};
     my $filter_scheme = $_{filter} || $_{filters} || $_{filter_scheme} || [];
     for my $rule (@$filter_scheme) {
         $self->filter(@$rule);
     }
 
+    if (my $minify = $_{minify}) {
+        if      ($minify eq 1 || $minify =~ m/^\s*(?:minifier-)?best\s*$/i)  { $self->filter("minifier-best") }
+        elsif   ($minify =~ m/^\s*yui-?compressor:/)                         { $self->filter($minify) }
+        elsif   ($minify =~ m/\.jar/i)                                       { $self->filter("yuicompressor:$minify") }
+        elsif   ($minify =~ m/^\s*(?:minifier-)?xs\s*$/i)                    { $self->filter("minifier-xs") }
+        elsif   ($minify =~ m/^\s*minifier\s*$/i)                            { $self->filter("minifier") }
+        elsif   ($minify =~ m/^\s*concat\s*$/i)                              { $self->filter("concat") }
+        else                                                                 { croak "Don't understand minify option ($minify)" }
+    }
+
     return $self;
 }
 
@@ -258,13 +353,9 @@ sub include {
     my %asset = @asset;
     _correct_for_proper_rank_and_type_order \%asset;
 
-    my $asset = File::Assets::Asset->new(base => $self->rsc, %asset);
+    my $asset = File::Assets::Asset->new(base => $self->rsc, cache => $self->cache, %asset);
 
-    return $self->fetch($asset->path) if $asset->path && $self->exists($asset->path);
-
-    $self->store($asset);
-
-    return $asset;
+    return $self->fetch_or_store($asset);
 }
 
 =head2 $asset = $assets->include_content(<content>, [ <type>, <rank>, { ... } ])
@@ -428,9 +519,9 @@ Returns true if <path> has been included, 0 otherwise.
 
 sub exists {
     my $self = shift;
-    my $path = shift;
+    my $key = shift;
 
-    return exists $self->_registry_hash->{$path} ? 1 : 0;
+    return exists $self->_registry_hash->{$key} ? 1 : 0;
 }
 
 =head2 $assets->store( <asset> )
@@ -443,7 +534,7 @@ sub store {
     my $self = shift;
     my $asset = shift;
 
-    $self->_registry_hash->{$asset->key} = $asset;
+    return $self->_registry_hash->{$asset->key} = $asset;
 }
 
 =head2 $asset = $assets->fetch( <path> )
@@ -461,6 +552,15 @@ sub fetch {
     return $self->_registry_hash->{$key};
 }
 
+sub fetch_or_store {
+    my $self = shift;
+    my $asset = shift;
+
+    return $self->fetch($asset->key) if $self->exists($asset->key);
+
+    return $self->store($asset);
+}
+
 sub kind {
     my $self = shift;
     my $asset = shift;
@@ -513,9 +613,126 @@ sub _exports {
     return map { $_->exports } @bucket;
 }
 
+=head2 $assets->set_name( <name> )
+
+Set the name of $assets
+
+This is exactly the same as
+
+    $assets->name( <name> )
+
+=cut
+
+
+=head2 $assets->set_base( <base> )
+
+Set the base uri, dir, and path for assets
+
+<base> can be a L<Path::Resource>, L<URI::ToDisk>, or a hash reference of the form:
+
+    { uri => ..., dir => ..., path => ... }
+
+Given a dir of C</var/www/htdocs>, a uri of C<http://example.com/static>, and a
+path of C<assets> then:
+
+    $assets will look for files in "/var/www/htdocs/assets"
+
+    $assets will "serve" files with "http://example.com/static/assets"
+
+=cut
+
+sub set_base {
+    my $self = shift;
+    croak "No base given" unless @_;
+    my $base = 1 == @_ ? shift : { @_ };
+    croak "No base given" unless $base;
+
+    $self->{rsc} = File::Assets::Util->parse_rsc($base);
+}
+
+=head2 $assets->set_base_uri( <uri> )
+
+Set the base uri for assets 
+
+=cut
+
+sub set_base_uri {
+    my $self = shift;
+    croak "No base uri given" unless defined $_[0];
+
+    $self->{rsc}->base->uri(shift);
+}
+
+=head2 $assets->set_base_dir( <dir> )
+
+Set the base dir for assets 
+
+=cut
+
+sub set_base_dir {
+    my $self = shift;
+    croak "No base dir given" unless defined $_[0];
+
+    $self->{rsc}->base->dir(shift);
+}
+
+=head2 $assets->set_base_path( <path> )
+
+Set the base path for assets 
+
+Passing an undefined value for <path> will clear/get-rid-of the path
+
+=cut
+
+sub set_base_path {
+    my $self = shift;
+    my $path;
+    $path = defined $_[0] ? Path::Abstract->new(shift) :  Path::Abstract->new;
+    # TODO-b This is very bad
+    $self->{rsc}->_path($path);
+}
+
 sub set_output_path_scheme {
     my $self = shift;
-    $self->{output_path_scheme} = shift;
+    my $scheme = shift;
+
+    if ($scheme && ref $scheme ne "ARRAY") {
+        $scheme = [ [ qw/*/ => $scheme ] ];
+    }
+
+    $self->{output_path_scheme} = $scheme;
+}
+
+=head2 $assets->set_output_path( <path> )
+
+Set the output path for assets generated by $assets
+
+See "Specifying an C<output_path> pattern" above
+
+=cut
+
+sub set_output_path {
+    my $self = shift;
+    $self->set_output_path_scheme(@_);
+}
+
+=head2 $assets->set_cache( <cache> )
+
+Specify the cache object or cache name to use
+
+=cut 
+
+sub set_cache {
+    my $self = shift;
+    my $cache = shift;
+
+    if ($cache) {
+        $cache = File::Assets::Cache->new(name => $cache) unless blessed $cache && $cache->isa("File::Assets::Cache");
+        $self->{cache} = $cache;
+    }
+    else {
+        delete $self->{cache};
+    }
 }
 
 sub filter {
@@ -535,7 +752,7 @@ sub filter {
 
     my $_filter = $filter;
     unless (blessed $_filter) {
-        croak "Couldn't find filter for ($filter)"  unless $_filter = File::Assets::Util->parse_filter($_filter, @_, assets => $self);
+        croak "Couldn't find filter for ($filter)" unless $_filter = File::Assets::Util->parse_filter($_filter, @_, assets => $self);
     }
 
     push @$category, $_filter;
@@ -570,7 +787,7 @@ sub _calculate_best {
     my ($best_kind, %return);
     %return = %$default if $default;
 
-    # TODO Cache the result of this
+    # TODO-f Cache the result of this
     for my $rule (@$scheme) {
         my ($condition, $action, $flags) = @$rule;
 
@@ -667,6 +884,16 @@ sub output_asset {
 
 Robert Krimen, C<< <rkrimen at cpan.org> >>
 
+=head1 SEE ALSO
+
+L<Catalyst::Plugin::Assets>
+
+L<Google::AJAX::Library>
+
+L<JS::YUI::Loader>
+
+L<JS::jQuery::Loader>
+
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-file-assets at rt.cpan.org>, or through
@@ -717,3 +944,20 @@ under the same terms as Perl itself.
 =cut
 
 1; # End of File::Assets
+
+__END__
+
+#    if (my $cache = $self->cache) {
+#        return 1 if $cache->exists($self->rsc->dir, $key);
+#    }
+
+#    if (my $cache = $self->cache) {
+#        $cache->store($self->rsc->dir, $asset);
+#    }
+
+#    if (my $cache = $self->cache) {
+#        if ($asset = $cache->fetch($self->rsc->dir, $key)) {
+#            return $self->store($asset);
+#        }
+#    }
+
